/*
 * 版权所有 (C) 2015 知启蒙(ZHIQIM) 保留所有权利。[遇见知启蒙，邂逅框架梦]
 * 
 * https://zhiqim.org/project/zhiqim_framework/zhiqim_kernel.htm
 *
 * Zhiqim Kernel is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2.
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PSL v2 for more details.
 */
package org.zhiqim.kernel.control;

import org.zhiqim.kernel.Service;

/**
 * 实现了线程的服务
 *
 * @ThreadController    线程控制器
 * @HandleController    处理控制器
 * @EventController     事件控制器
 * @version v1.0.0 @author zouzhigang 2018-11-20 新建与整理
 */
public abstract class ThreadServicer extends Threadx implements Service
{
    protected String id;
    protected int index;
    protected ClassLoader classLoader;
    
    /*********************************************************/
    //服务编号和索引引
    /*********************************************************/
    
    /** 设置服务编号,对应conf/zhiqim.xml/service/key */
    public final void setId(String id)
    {
        this.id = id;
    }

    /** 获取服务编号 */
    public final String getId()
    {
        return id;
    }
    
    /** 设置全局变量中多例模式中的索引号 */
    public void setIndex(int index)
    {
        this.index = index;
    }
    
    /** 获取全局变量中多例模式中的索引号 */
    public int getIndex()
    {
        return this.index;
    }
    
    /** 获取类加载器 */
    public ClassLoader getClassLoader()
    {
        if (classLoader != null)
            return classLoader;
        return this.getClass().getClassLoader();
    }

    /** 设置类加载器 */
    public void setClassLoader(ClassLoader classLoader)
    {
        this.classLoader = classLoader;
    }
    
    /*********************************************************/
    //服务创建和销毁
    /*********************************************************/
    
    /**
     * 服务创建接口，配置信息从Global中读取，子类必须实现
     * 
     * @return boolean =true,启动成功,=false,启动失败
     */
    public boolean create()
    {
        return open();
    }
    
    /**
     * 服务销毁接口，配置信息从Global中读取，默认不处理，子类可选重写
     * 
     * @throws Exception Hook捕捉,打印日志,异常不处理，直接退出
     */
    public void destroy()
    {
        close();
    }
    
    
    /*********************************************************/
    //更新和测试更新配置
    /*********************************************************/
    
    /**
     * 测试更新配置，默认支持更新配置
     * 
     * @return              =true表示可以更新，=false表示更新可能有问题
     * @throws Exception    异常时表示不可更新
     */
    public boolean test() throws Exception
    {
        return false;
    }
    
    /**
     * 服务更新配置，默认不处理更新配置
     * 
     * @throws Exception 出现更新异常通知配置端
     */
    public void update() throws Exception
    {
    }
}
