/*
 * 版权所有 (C) 2015 知启蒙(ZHIQIM) 保留所有权利。[遇见知启蒙，邂逅框架梦]
 * 
 * https://zhiqim.org/project/zhiqim_framework/zhiqim_kernel.htm
 *
 * Zhiqim Kernel is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2.
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PSL v2 for more details.
 */
package org.zhiqim.kernel.json;

/**
 * JSON 主程序，对象转换成JSON和JSON转换成对象
 *
 * @version v1.0.0 @author zouzhigang 2014-3-21 新建与整理
 */
public class Json
{
    public static final Json json = new Json();
    
    private boolean isJsLongToString = true;                //是否适配JS中超过2^53时long转为String
    private JsonFactory factory = new JsonFactory(this);
    
    /**
     * 对象转换成JSON字符串
     * 
     * @param obj   对象
     * @return      JSON字符串
     */
    public String toString(Object obj)
    {
        if (obj == null)
            return "null";
        
        return factory.getParser(obj.getClass()).toString(obj);
    }
    
    /**
     * JSON字符串转换成对象
     * 
     * @param json      JSON字符串
     * @param cls       转换成对象的类
     * @return          obj对象
     */
    public <T> T toObject(String json, Class<T> cls)
    {
        return toObject(json, cls, true);
    }
    
    /**
     * JSON字符串转换成对象
     * 
     * @param json      JSON字符串
     * @param cls     转换成对象的类
     * @param chkSpace  是否检查空格
     * @return          obj对象
     */
    @SuppressWarnings("unchecked")
    public <T> T toObject(String json, Class<T> cls, boolean chkSpace)
    {
        if (json == null || cls == null)
            return null;
        
        if (chkSpace)
            json = Jsons.removeBlankSpace(json);
        
        if ("null".equals(json))
            return null;
        
        return (T)factory.getParser(cls).toObject(json, cls);
    }

    /***************************************************************/
    //以下为JSON定义的解析参数
    /***************************************************************/
    
    public boolean isJsLong()
    {
        return isJsLongToString;
    }

    public Json setJsLongToString(boolean isJsLongToString)
    {
        this.isJsLongToString = isJsLongToString;
        return this;
    }
}
