/*
 * 版权所有 (C) 2015 知启蒙(ZHIQIM) 保留所有权利。[遇见知启蒙，邂逅框架梦]
 * 
 * https://zhiqim.org/project/zhiqim_framework/zhiqim_kernel.htm
 *
 * Zhiqim Kernel is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2.
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PSL v2 for more details.
 */
package org.zhiqim.kernel.json;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import org.zhiqim.kernel.extend.HashMapCV;
import org.zhiqim.kernel.extend.HashMapSO;
import org.zhiqim.kernel.extend.HashMapSS;
import org.zhiqim.kernel.extend.HashMapSV;
import org.zhiqim.kernel.extend.LinkedMapSO;
import org.zhiqim.kernel.extend.LinkedMapSS;
import org.zhiqim.kernel.extend.LinkedMapSV;
import org.zhiqim.kernel.extend.MapSO;
import org.zhiqim.kernel.extend.MapSS;
import org.zhiqim.kernel.extend.MapSV;
import org.zhiqim.kernel.extend.TreeMapSO;
import org.zhiqim.kernel.extend.TreeMapSS;
import org.zhiqim.kernel.extend.TreeMapSV;
import org.zhiqim.kernel.json.parser.ArrayParser;
import org.zhiqim.kernel.json.parser.BaseParser;
import org.zhiqim.kernel.json.parser.DateParser;
import org.zhiqim.kernel.json.parser.JsonStringParser;
import org.zhiqim.kernel.json.parser.ListParser;
import org.zhiqim.kernel.json.parser.MapParser;
import org.zhiqim.kernel.json.parser.MapSOParser;
import org.zhiqim.kernel.json.parser.MapSSParser;
import org.zhiqim.kernel.json.parser.MapSVParser;
import org.zhiqim.kernel.json.parser.ObjectParser;
import org.zhiqim.kernel.util.Classes;

/**
 * Json解析适配工厂类
 *
 * @version v1.0.0 @author zouzhigang 2014-3-21 新建与整理
 */
public class JsonFactory
{
    private HashMapCV<JsonParser> parserMap;
    
    public JsonFactory(Json jsonMain)
    {
        parserMap = new HashMapCV<JsonParser>();
        
        //8种基本类型和字符串
        JsonParser baseParser = new BaseParser().set(jsonMain);
        parserMap.put(boolean.class, baseParser);
        parserMap.put(byte.class, baseParser);
        parserMap.put(char.class, baseParser);
        parserMap.put(short.class, baseParser);
        parserMap.put(int.class, baseParser);
        parserMap.put(long.class, baseParser);
        parserMap.put(float.class, baseParser);
        parserMap.put(double.class, baseParser);
        parserMap.put(Boolean.class, baseParser);
        parserMap.put(Byte.class, baseParser);
        parserMap.put(Character.class, baseParser);
        parserMap.put(Short.class, baseParser);
        parserMap.put(Integer.class, baseParser);
        parserMap.put(Long.class, baseParser);
        parserMap.put(Float.class, baseParser);
        parserMap.put(Double.class, baseParser);
        parserMap.put(String.class, baseParser);
        
        //特殊字符串
        JsonParser jsParser = new JsonStringParser();
        parserMap.put(JsonString.class, jsParser);
        
        //数组类型，8种基本类型数组、字符串数组和对象数组
        JsonParser arrarParser = new ArrayParser().set(jsonMain);
        parserMap.put(boolean[].class, arrarParser);
        parserMap.put(byte[].class, arrarParser);
        parserMap.put(char[].class, arrarParser);
        parserMap.put(short[].class, arrarParser);
        parserMap.put(int[].class, arrarParser);
        parserMap.put(long[].class, arrarParser);
        parserMap.put(float[].class, arrarParser);
        parserMap.put(double[].class, arrarParser);
        parserMap.put(Boolean[].class, arrarParser);
        parserMap.put(Byte[].class, arrarParser);
        parserMap.put(Character[].class, arrarParser);
        parserMap.put(Short[].class, arrarParser);
        parserMap.put(Integer[].class, arrarParser);
        parserMap.put(Long[].class, arrarParser);
        parserMap.put(Float[].class, arrarParser);
        parserMap.put(Double[].class, arrarParser);
        parserMap.put(String[].class, arrarParser);
        
        //哈希表
        JsonParser mapParser = new MapParser().set(jsonMain);
        parserMap.put(Map.class, mapParser);
        parserMap.put(HashMap.class, mapParser);
        parserMap.put(ConcurrentMap.class, mapParser);
        parserMap.put(ConcurrentHashMap.class, mapParser);
        parserMap.put(Hashtable.class, mapParser);
        parserMap.put(LinkedHashMap.class, mapParser);
        parserMap.put(TreeMap.class, mapParser);
        
        //知启蒙哈希表
        JsonParser zmapParser = new MapSOParser().set(jsonMain);
        parserMap.put(MapSO.class, zmapParser);
        parserMap.put(HashMapSO.class, zmapParser);
        parserMap.put(LinkedMapSO.class, zmapParser);
        parserMap.put(TreeMapSO.class, zmapParser);
        
        JsonParser zmapssParser = new MapSSParser().set(jsonMain);
        parserMap.put(MapSS.class, zmapssParser);
        parserMap.put(HashMapSS.class, zmapssParser);
        parserMap.put(LinkedMapSS.class, zmapssParser);
        parserMap.put(TreeMapSS.class, zmapssParser);
        
        JsonParser zmapsvParser = new MapSVParser().set(jsonMain);
        parserMap.put(MapSV.class, zmapsvParser);
        parserMap.put(HashMapSV.class, zmapsvParser);
        parserMap.put(LinkedMapSV.class, zmapsvParser);
        parserMap.put(TreeMapSV.class, zmapsvParser);
        
        //链表
        JsonParser listParser = new ListParser().set(jsonMain);
        parserMap.put(Collection.class, listParser);
        parserMap.put(List.class, listParser);
        parserMap.put(ArrayList.class, listParser);
        parserMap.put(LinkedList.class, listParser);
        parserMap.put(Set.class, listParser);
        parserMap.put(HashSet.class, listParser);
        
        //日期对象
        JsonParser dateParser = new DateParser().set(jsonMain);
        parserMap.put(java.util.Date.class, dateParser);
        parserMap.put(java.sql.Date.class, dateParser);
        parserMap.put(java.sql.Time.class, dateParser);
        parserMap.put(java.sql.Timestamp.class, dateParser);
        parserMap.put(Calendar.class, dateParser);
        
        //最后放入通用对象和对象数组解析器
        parserMap.put(Object[].class, arrarParser);
        parserMap.put(Object.class, new ObjectParser().set(jsonMain));
    }
    
    /** 根据类获取解析器 */
    public JsonParser getParser(Class<?> cls)
    {
        JsonParser parser = parserMap.get(cls);
        if (parser != null)
            return parser;
        
        //列表/哈希表/数组，其他的认为是对象
        if (Classes.isCollection(cls))
            return parserMap.get(List.class);
        else if (Classes.isMap(cls))
            return parserMap.get(Map.class);
        else if (cls.isArray())
            return parserMap.get(Object[].class);
        else
            return parserMap.get(Object.class);
    }
}
