/*
 * 版权所有 (C) 2015 知启蒙(ZHIQIM) 保留所有权利。[遇见知启蒙，邂逅框架梦]
 * 
 * https://zhiqim.org/project/zhiqim_framework/zhiqim_kernel.htm
 *
 * Zhiqim Kernel is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2.
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PSL v2 for more details.
 */
package org.zhiqim.kernel.json.parser;

import java.lang.reflect.Array;
import java.util.List;

import org.zhiqim.kernel.constants.SignConstants;
import org.zhiqim.kernel.json.Json;
import org.zhiqim.kernel.json.JsonException;
import org.zhiqim.kernel.json.JsonParser;
import org.zhiqim.kernel.json.Jsons;
import org.zhiqim.kernel.util.Arrays;
import org.zhiqim.kernel.util.Ints;
import org.zhiqim.kernel.util.Longs;
import org.zhiqim.kernel.util.Validates;

/**
 * 数组解析器,8种基本类型数组、字符串数组和对象数组
 *
 * @version v1.0.0 @author zouzhigang 2014-3-21 新建与整理
 */
public class ArrayParser implements JsonParser, SignConstants
{
    private Json jsonMain;
    
    @Override
    public JsonParser set(Json jsonMain)
    {
        this.jsonMain = jsonMain;
        return this;
    }
    
    @Override
    public String toString(Object obj)
    {
        Object[] objs = Arrays.toArray(obj);
        
        StringBuilder strb = new StringBuilder("[");
        for (int i=0;i<objs.length;i++)
        {
            Object o = objs[i];
            if (i > 0)
                strb.append(",").append(jsonMain.toString(o));
            else
                strb.append(jsonMain.toString(o));
        }
        strb.append("]");
        return strb.toString();
    }

    @Override
    public Object toObject(String json, Class<?> cls)
    {
        List<String> list = Jsons.toListByArray(json);
        
        if (cls == boolean[].class)       //1(0|1)
            return toBooleans(list);
        else if (cls == Boolean[].class)
            return toBooleanObjs(list);
        else if (cls == byte[].class)     //1
            return toBytes(list);
        else if (cls == Byte[].class)
            return toByteObjs(list);
        else if (cls == char[].class)     //1,2,3
            return toChars(list);
        else if (cls == Character[].class)
            return toCharObjs(list);
        else if (cls == short[].class)    //2
            return toShorts(list);
        else if (cls == Short[].class)
            return toShortObjs(list);
        else if (cls == int[].class)      //4
            return toInts(list);
        else if (cls == Integer[].class)
            return toIntObjs(list);
        else if (cls == long[].class)     //8
            return toLongs(list);
        else if (cls == Long[].class)
            return toLongObjs(list);
        else if (cls == float[].class)    //4
            return toFloats(list);
        else if (cls == Float[].class)
            return toFloatObjs(list);
        else if (cls == double[].class)   //8
            return toDoubles(list);
        else if (cls == Double[].class)
            return toDoubleObjs(list);
        else if (cls == String[].class)   //字符串
            return toStrings(list);
        else
        {
            //非基本类型，则使用对象
            Class<?> type = cls.getComponentType();
            Object[] objs = (Object[])Array.newInstance(type, list.size());
            for (int i=0;i<list.size();i++)
            {
                objs[i] = (jsonMain.toObject(list.get(i), type, false));
            }
            
            return objs;
        }
    }

    private boolean[] toBooleans(List<String> list)
    {
        boolean[] val = new boolean[list.size()];
        for (int i=0;i<val.length;i++)
        {
            String v = Jsons.removeStartEndQuotation(list.get(i));
            if (_TRUE_.equals(v))
                val[i] = true;
            else if (_FALSE_.equals(v))
                val[i] = false;
            else
                throw new JsonException("Json[解析布尔型]时出现非[true|false]的字符["+v+"]");
        }
        
        return val;
    }
    
    private Boolean[] toBooleanObjs(List<String> list)
    {
        Boolean[] val = new Boolean[list.size()];
        for (int i=0;i<val.length;i++)
        {
            String v = Jsons.removeStartEndQuotation(list.get(i));
            if (_TRUE_.equals(v))
                val[i] = true;
            else if (_FALSE_.equals(v))
                val[i] = false;
            else
                throw new JsonException("Json[解析boolean]时出现非[true|false]的字符["+v+"]");
        }
        
        return val;
    }
    
    private byte[] toBytes(List<String> list)
    {
        byte[] val = new byte[list.size()];
        for (int i=0;i<val.length;i++)
        {
            String v = Jsons.removeStartEndQuotation(list.get(i));
            try
            {
                val[i] = Byte.parseByte(v);
            }
            catch(NumberFormatException e)
            {
                throw new JsonException("Json[解析byte]时出现非数值型的字符["+v+"]");
            }
        }
        
        return val;
    }
    
    private Byte[] toByteObjs(List<String> list)
    {
        Byte[] val = new Byte[list.size()];
        for (int i=0;i<val.length;i++)
        {
            String v = Jsons.removeStartEndQuotation(list.get(i));
            try
            {
                val[i] = Byte.parseByte(v);
            }
            catch(NumberFormatException e)
            {
                throw new JsonException("Json[解析byte]时出现非数值型的字符["+v+"]");
            }
        }
        
        return val;
    }
    
    private char[] toChars(List<String> list)
    {
        char[] val = new char[list.size()];
        for (int i=0;i<val.length;i++)
        {
            String v = Jsons.removeStartEndQuotation(list.get(i));
            if (Validates.isEmpty(v))
                throw new JsonException("Json[解析char]时出现非字符型的字符["+v+"]");
            
            val[i] = v.charAt(0);
        }
        
        return val;
    }
    
    private Character[] toCharObjs(List<String> list)
    {
        Character[] val = new Character[list.size()];
        for (int i=0;i<val.length;i++)
        {
            String v = Jsons.removeStartEndQuotation(list.get(i));
            if (Validates.isEmpty(v))
                throw new JsonException("Json[解析char]时出现非字符型的字符["+v+"]");
            
            val[i] = v.charAt(0);
        }
        
        return val;
    }
    
    private short[] toShorts(List<String> list)
    {
        short[] val = new short[list.size()];
        for (int i=0;i<val.length;i++)
        {
            String v = Jsons.removeStartEndQuotation(list.get(i));
            try
            {
                val[i] = Short.parseShort(v);
            }
            catch(NumberFormatException e)
            {
                throw new JsonException("Json[解析short]时出现非数值型的字符["+v+"]");
            }
        }
        
        return val;
    }
    
    private Short[] toShortObjs(List<String> list)
    {
        Short[] val = new Short[list.size()];
        for (int i=0;i<val.length;i++)
        {
            String v = Jsons.removeStartEndQuotation(list.get(i));
            try
            {
                val[i] = Short.parseShort(v);
            }
            catch(NumberFormatException e)
            {
                throw new JsonException("Json[解析short]时出现非数值型的字符["+v+"]");
            }
        }
        
        return val;
    }
    
    private int[] toInts(List<String> list)
    {
        int[] val = new int[list.size()];
        for (int i=0;i<val.length;i++)
        {
            String v = Jsons.removeStartEndQuotation(list.get(i));
            try
            {
                val[i] = Ints.toInt(v);
            }
            catch(NumberFormatException e)
            {
                throw new JsonException("Json[解析int]时出现非数值型的字符["+v+"]");
            }
        }
        
        return val;
    }
    
    private Integer[] toIntObjs(List<String> list)
    {
        Integer[] val = new Integer[list.size()];
        for (int i=0;i<val.length;i++)
        {
            String v = Jsons.removeStartEndQuotation(list.get(i));
            try
            {
                val[i] = Ints.toInt(v);
            }
            catch(NumberFormatException e)
            {
                throw new JsonException("Json[解析int]时出现非数值型的字符["+v+"]");
            }
        }
        
        return val;
    }
    
    private long[] toLongs(List<String> list)
    {
        long[] val = new long[list.size()];
        for (int i=0;i<val.length;i++)
        {
            String v = Jsons.removeStartEndQuotation(list.get(i));
            try
            {
                val[i] = Longs.toLong(v);
            }
            catch(NumberFormatException e)
            {
                throw new JsonException("Json[解析long]时出现非数值型的字符["+v+"]");
            }
        }
        
        return val;
    }
    
    private Long[] toLongObjs(List<String> list)
    {
        Long[] val = new Long[list.size()];
        for (int i=0;i<val.length;i++)
        {
            String v = Jsons.removeStartEndQuotation(list.get(i));
            try
            {
                val[i] = Longs.toLong(v);
            }
            catch(NumberFormatException e)
            {
                throw new JsonException("Json[解析long]时出现非数值型的字符["+v+"]");
            }
        }
        
        return val;
    }
    
    private float[] toFloats(List<String> list)
    {
        float[] val = new float[list.size()];
        for (int i=0;i<val.length;i++)
        {
            String v = Jsons.removeStartEndQuotation(list.get(i));
            try
            {
                val[i] = Float.parseFloat(v);
            }
            catch(NumberFormatException e)
            {
                throw new JsonException("Json[解析float]时出现非数值型的字符["+v+"]");
            }
        }
        
        return val;
    }
    
    private Float[] toFloatObjs(List<String> list)
    {
        Float[] val = new Float[list.size()];
        for (int i=0;i<val.length;i++)
        {
            String v = Jsons.removeStartEndQuotation(list.get(i));
            try
            {
                val[i] = Float.parseFloat(v);
            }
            catch(NumberFormatException e)
            {
                throw new JsonException("Json[解析float]时出现非数值型的字符["+v+"]");
            }
        }
        
        return val;
    }
    
    private double[] toDoubles(List<String> list)
    {
        double[] val = new double[list.size()];
        for (int i=0;i<val.length;i++)
        {
            String v = Jsons.removeStartEndQuotation(list.get(i));
            try
            {
                val[i] = Double.parseDouble(v);
            }
            catch(NumberFormatException e)
            {
                throw new JsonException("Json[解析double]时出现非数值型的字符["+v+"]");
            }
        }
        
        return val;
    }
    
    private Double[] toDoubleObjs(List<String> list)
    {
        Double[] val = new Double[list.size()];
        for (int i=0;i<val.length;i++)
        {
            String v = Jsons.removeStartEndQuotation(list.get(i));
            try
            {
                val[i] = Double.parseDouble(v);
            }
            catch(NumberFormatException e)
            {
                throw new JsonException("Json[解析double]时出现非数值型的字符["+v+"]");
            }
        }
        
        return val;
    }
    
    private String[] toStrings(List<String> list)
    {
        String[] val = new String[list.size()];
        for (int i=0;i<val.length;i++)
        {
            String v = Jsons.removeStartEndQuotation(list.get(i));
            //字符串情况，要对转义作删除操作
            val[i] = Jsons.removeEscapeChar(v);
        }
        
        return val;
    }
}
