/*
 * 版权所有 (C) 2015 知启蒙(ZHIQIM) 保留所有权利。[遇见知启蒙，邂逅框架梦]
 * 
 * https://zhiqim.org/project/zhiqim_framework/zhiqim_kernel.htm
 *
 * Zhiqim Kernel is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2.
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PSL v2 for more details.
 */
package org.zhiqim.kernel.tree;

import java.util.List;

import org.zhiqim.kernel.util.Asserts;
import org.zhiqim.kernel.util.Validates;

/**
 * 树生成器
 *
 * @version v1.0.0 @author zouzhigang 2017-8-12 新建与整理
 */
public class TreeBuilder
{
    /**
     * 生成新的树型模式
     * 
     * @param list          列表
     * @param treer         判断器
     * @return              树对象
     */
    public static <E> Tree<E> newTree(List<E> list, Treer<E> treer)
    {
        Asserts.as(Validates.isNotEmpty(list)?null:"树型对象要求列表不能为空");
        Asserts.as(treer != null?null:"树型对象要求判断器不能为空");
        
        return new Tree<E>(list.get(0), treer).build(list);
    }
    
    /**
     * 生成新的树型模式
     * 
     * @param list          列表
     * @param pFieldName    父字段方法名
     * @param sFieldName    自字段方法名
     * @return              树对象
     */
    public static <E> Tree<E> newTree(List<E> list, String pFieldName, String sFieldName)
    {
        Asserts.as(Validates.isNotEmpty(list)?null:"树型对象要求列表不能为空");
        return new Tree<E>(list.get(0), new TreeFielder<E>(pFieldName, sFieldName)).build(list);
    }
}
